@@ -2,6 +2,7 @@
 #include <config.h>
 
 #include <string.h>
+#include <libgnomecups/gnome-cups-ui-driver.h>
 
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkhbox.h>
@@ -154,19 +155,29 @@ druid_update_sensitivities (GladeXML *xml)
 void
 druid_watch_sensitivity_widget (GladeXML *xml, const char *name)
 {
-	GtkWidget *widget = glade_xml_get_widget (xml, name);
-	if (GTK_IS_TOGGLE_BUTTON (widget))
-		g_signal_connect_swapped (widget,
+	GtkWidget *w = glade_xml_get_widget (xml, name);
+	if (GTK_IS_TOGGLE_BUTTON (w))
+		g_signal_connect_swapped (w,
 			"toggled",
 			G_CALLBACK (druid_update_sensitivities), xml);
-	else if (GTK_IS_EDITABLE (widget) || GTK_IS_OPTION_MENU (widget))
-		g_signal_connect_swapped (widget, "changed",
-			G_CALLBACK (druid_update_sensitivities), xml);
-	else if (GTK_IS_TREE_VIEW (widget))
-		g_signal_connect_swapped (
-			gtk_tree_view_get_selection (GTK_TREE_VIEW (widget)),
+	else if (GTK_IS_EDITABLE (w)
+		 || GTK_IS_COMBO_BOX (w)
+		 || IS_GCUPS_DRIVER_SELECTOR(w)) {
+		g_signal_connect_swapped (w,
 			"changed",
 			G_CALLBACK (druid_update_sensitivities), xml);
+		if (GTK_IS_COMBO_BOX_ENTRY (w))
+		/* cheesy hack, we should be able to do this more gracefully */
+			g_signal_connect_swapped (gtk_bin_get_child (GTK_BIN (w)),
+				"changed",
+				G_CALLBACK (druid_update_sensitivities), xml);
+	} else if (GTK_IS_TREE_VIEW (w))
+		g_signal_connect_swapped (
+			gtk_tree_view_get_selection (GTK_TREE_VIEW (w)),
+			"changed",
+			G_CALLBACK (druid_update_sensitivities), xml);
+	else
+		g_warning ("unknown widget %s", name);
 }
 
 gboolean 
@@ -184,9 +195,7 @@ tree_view_has_selection (GladeXML *xml, const char *name)
 gboolean 
 toggle_button_is_active (GladeXML *xml, const char *name)
 {
-	GtkWidget *widget;
-	
-	widget = glade_xml_get_widget (xml, name);
+	GtkWidget *widget = glade_xml_get_widget (xml, name);
 	return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 }
 
