@@ -26,26 +26,20 @@
 #include "view-printers.h"
 #include "view-queue.h"
 #include "printer-properties.h"
-#include "util.h"
 
-#include <cups/cups.h>
-
-#include <stdlib.h>
-#include <bonobo/bonobo-exception.h>
-#include <bonobo/bonobo-listener.h>
-#include <bonobo/bonobo-main.h>
 #include <glade/glade.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmessagedialog.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-client.h>
-#include <libgnomeui/gnome-ui-init.h>
-#include <libgnomeui/gnome-icon-theme.h>
 #include <libgnomecups/gnome-cups-util.h>
 #include <libgnomecups/gnome-cups-ui-init.h>
 #include <libgnomecups/gnome-cups-printer.h>
 #include <libgnomecups/gnome-cups-request.h>
 #include <libgnomecups/gnome-cups-ui-print.h>
+#include <libgnomecups/gnome-cups-ui-util.h>
+
+#include <glib/gi18n.h>
+#include <stdlib.h>
+#include <cups/cups.h>
 
 static void
 printer_view_weak_notify (gpointer user_data, GObject *old_spot)
